from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, MiddlewareNotUsed
from django.db.models import Q
from django.http import Http404, HttpResponsePermanentRedirect
from django.middleware.security import SecurityMiddleware

from feincms3.apps import (
    AppsMiddleware as BaseMiddleware, AppsMixin, apps_urlconf,
)
from feincms3.utils import concrete_model

from .models import Site


def apps_urlconf_for_site(site):
    page_model = concrete_model(AppsMixin)
    fields = (
        'path', 'application', 'app_instance_namespace', 'language_code',
    )
    apps = page_model.objects.active().filter(
        Q(site=site),
        ~Q(app_instance_namespace=''),
    ).values_list(*fields).order_by(*fields)
    return apps_urlconf(apps=apps)


class SiteMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        request.site = Site.objects.for_host(request.get_host())
        if request.site is None:
            raise Http404('No configuration found for %r' % request.get_host())
        return self.get_response(request)


class AppsMiddleware(BaseMiddleware):
    def __call__(self, request):
        request.site = Site.objects.for_host(request.get_host())
        if request.site is None:
            raise Http404('No configuration found for %r' % request.get_host())
        request.urlconf = apps_urlconf_for_site(request.site)
        return self.get_response(request)


class CanonicalDomainMiddleware(SecurityMiddleware):
    canonical_domain_secure = None

    def __init__(self, get_response):
        super().__init__(get_response)
        if settings.DEBUG:
            raise MiddlewareNotUsed

        self.canonical_domain_secure = getattr(
            settings,
            'CANONICAL_DOMAIN_SECURE',
            False,
        )

    def process_request(self, request):
        if not hasattr(request, 'site'):
            raise ImproperlyConfigured(
                'No "site" attribute on request. Insert SiteMiddleware'
                ' or AppsMiddleware before CanonicalDomainMiddleware.'
            )

        matches = request.get_host() == request.site.host
        if matches and self.canonical_domain_secure and request.is_secure():
            return
        elif matches and not self.canonical_domain_secure:
            return

        return HttpResponsePermanentRedirect('http%s://%s%s' % (
            's' if (
                self.canonical_domain_secure or request.is_secure()
            ) else '',
            request.site,
            request.get_full_path(),
        ))



# Generated by Django 2.0.4 on 2018-04-12 09:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('feincms3_sites', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='site',
            name='is_managed_re',
            field=models.BooleanField(default=True, help_text='Deactivate this to specify the regex yourself.', verbose_name='manage the host regex'),
        ),
    ]



# Generated by Django 2.0.4 on 2018-04-12 09:25

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Site',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_default', models.BooleanField(default=False, verbose_name='is default')),
                ('host', models.CharField(max_length=200, verbose_name='host')),
                ('host_re', models.CharField(blank=True, max_length=200, verbose_name='host regular expression')),
            ],
            options={
                'verbose_name': 'site',
                'verbose_name_plural': 'sites',
            },
        ),
    ]



import re

from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.db import models
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _

from feincms3.pages import AbstractPage as BasePage


class SiteQuerySet(models.QuerySet):
    """
    Return a site instance for the passed host, or ``None`` if there is no
    match and no default site.

    The default site's host regex is tested first.
    """
    def for_host(self, host):
        default = None
        for site in self.order_by('-is_default'):
            if re.match(site.host_re, host):
                return site
            elif site.is_default:
                default = site
        return default


class Site(models.Model):
    is_default = models.BooleanField(
        _('is default'),
        default=False,
    )
    host = models.CharField(
        _('host'),
        max_length=200,
    )
    is_managed_re = models.BooleanField(
        _('manage the host regex'),
        default=True,
        help_text=_('Deactivate this to specify the regex yourself.'),
    )
    host_re = models.CharField(
        _('host regular expression'),
        max_length=200,
        blank=True,
    )

    objects = SiteQuerySet.as_manager()

    class Meta:
        verbose_name = _('site')
        verbose_name_plural = _('sites')

    def __str__(self):
        return self.host

    def clean(self):
        if self.is_default:
            if self.__class__._base_manager.filter(
                Q(is_default=True),
                ~Q(pk=self.pk),
            ).exists():
                raise ValidationError(
                    _('Only one site can be the default site.'),
                )

    def save(self, *args, **kwargs):
        if self.is_managed_re:
            self.host_re = r'^%s$' % re.escape(self.host)
        super().save(*args, **kwargs)
    save.alters_data = True


class SiteForeignKey(models.ForeignKey):
    """
    The site foreign key field should not be required, so that we can fill in
    a value from the parent.
    """
    def formfield(self, **kwargs):
        kwargs['required'] = False
        return super().formfield(**kwargs)


class AbstractPage(BasePage):
    site = SiteForeignKey(
        Site,
        on_delete=models.CASCADE,
        verbose_name=_('site'),
        related_name='+',
    )
    # Exactly the same as BasePage.path,
    # except that it is not unique:
    path = models.CharField(
        _('path'),
        max_length=1000,
        blank=True,
        help_text=_('Generated automatically if \'static path\' is unset.'),
        validators=[
            RegexValidator(
                regex=r'^/(|.+/)$',
                message=_('Path must start and end with a slash (/).'),
            ),
        ],
    )

    class Meta:
        abstract = True
        unique_together = (('site', 'path'),)
        verbose_name = _('page')
        verbose_name_plural = _('pages')

    def clean_fields(self, exclude=None):
        """
        Since the ``SiteForeignKey`` adds ``required=False`` we have to add
        our own check here.
        """
        exclude = [] if exclude is None else exclude
        super().clean_fields(exclude)

        if not self.site_id and not self.parent_id:
            # Using validation_error() does not work as it should, because
            # 'site' is always part of exclude, because the model field is
            # required, but the form field is not, therefore Django adds
            # 'site' into 'exclude' unconditionally.
            raise ValidationError(_(
                'The site is required when creating root nodes.'
            ))

    def save(self, *args, **kwargs):
        if self.parent_id and self.parent.site_id:
            self.site_id = self.parent.site_id
        super().save(*args, **kwargs)
    save.alters_data = True



from django.apps import AppConfig
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _


class SitesAppConfig(AppConfig):
    name = 'feincms3_sites'
    verbose_name = capfirst(_('sites'))



from django.contrib import admin
from django.utils.text import capfirst
from django.utils.translation import gettext_lazy as _

from .models import Site


@admin.register(Site)
class SiteAdmin(admin.ModelAdmin):
    fieldsets = [
        (None, {
            'fields': ('is_default', 'host'),
        }),
        (capfirst(_('advanced')), {
            'fields': ('is_managed_re', 'host_re',),
            'classes': ('collapse',),
        }),
    ]
    list_display = ('host', 'is_default', 'is_managed_re', 'host_re')
    ordering = ('-is_default',)
